@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: InetAddrMask.cpp 243 2009-01-10 23:58:22Z vadim $
+  $Id$
 
 
   This program is free software which we release under the GNU General Public
@@ -218,7 +218,9 @@ bool libfwbuilder::_convert_range_to_networks(const InetAddr &start,
     if (end < start) return false;
     if (start == end)
     {
-	res.push_back(InetAddrMask(start, InetAddr(InetAddr::getAllOnes())));
+	res.push_back(InetAddrMask(
+                          start,
+                          InetAddr(InetAddr::getAllOnes(start.addressFamily()))));
 	return false;
     }
 
@@ -232,8 +234,12 @@ bool libfwbuilder::_convert_range_to_networks(const InetAddr &start,
 
     if (size==2)
     {
-	res.push_back(InetAddrMask(start, InetAddr(InetAddr::getAllOnes())));
-	res.push_back(InetAddrMask(end, InetAddr(InetAddr::getAllOnes())));
+	res.push_back(InetAddrMask(
+                          start,
+                          InetAddr(InetAddr::getAllOnes(start.addressFamily()))));
+	res.push_back(InetAddrMask(
+                          end,
+                          InetAddr(InetAddr::getAllOnes(end.addressFamily()))));
 	return false;
     }
 
@@ -313,8 +319,8 @@ vector<InetAddrMask> libfwbuilder::getOverlap(const InetAddrMask &n1,
     const InetAddr& s1 = *(n1.getAddressPtr());
     const InetAddr& s2 = *(n2.getAddressPtr());
 
-    const InetAddr&   m1 = *(n1.getNetmaskPtr());
-    const InetAddr&   m2 = *(n2.getNetmaskPtr());
+    const InetAddr& m1 = *(n1.getNetmaskPtr());
+    const InetAddr& m2 = *(n2.getNetmaskPtr());
 
     InetAddr e1 = s1 | (~m1);
     InetAddr e2 = s2 | (~m2);
@@ -327,34 +333,55 @@ vector<InetAddrMask> libfwbuilder::getOverlap(const InetAddrMask &n1,
  * 255.255.255.255). Check for this condition and replace e1 or e2
  * accordingly if needed.
  *
- * TODO: comparison with InetAddr() looks ugly. May be need to
- * overload operator==(int) so we could compare with '0'
- *
  */
-    if (s1==InetAddr() && m1==InetAddr()) e1=InetAddr(InetAddr::getAllOnes());
-    if (s2==InetAddr() && m2==InetAddr()) e2=InetAddr(InetAddr::getAllOnes());
+    if (s1.isAny() && m1.isAny())
+        e1 = InetAddr(InetAddr::getAllOnes(s1.addressFamily()));
+    if (s2.isAny() && m2.isAny())
+        e2 = InetAddr(InetAddr::getAllOnes(s2.addressFamily()));
 
     vector<InetAddrMask>  res;
 
-    InetAddr rs,re;
+    if (e2 < s1) return res;
+    if (e1 < s2) return res;
 
-    if (e2<s1) return res;
-    if (s2<s1 && e2>s1 && e2<e1) { rs=s1; re=e2; }
-    if (s2>s1 && e2<e1)          { rs=s2; re=e2; }
-    if (s2>s1 && s2<e1 && e2>e1) { rs=s2; re=e1; }
-    if (s2>e1) return res;
-    if (s2<s1 && e2>e1)          { rs=s1; re=e1; }
-    if (s1==s2 && e1==e2)        { rs=s1; re=e1; }
+    if (s1 == s2 && e1 == e2)
+    {
+        // libfwbuilder::_convert_range_to_networks(s1, e1, res);
+        res.push_back(n1);
+        return res;
+    }
+
+    if (e2 == s1)
+    {
+        res.push_back(InetAddrMask(s1, InetAddr::getAllOnes(s1.addressFamily())));
+        return res;
+    }
+
+    if (e1 == s2)
+    {
+        res.push_back(InetAddrMask(s2, InetAddr::getAllOnes(s2.addressFamily())));
+        return res;
+    }
+
+    InetAddr rs, re;
+
+    if (s1 == s2) { rs = s1; }
+    if (s1 < s2)  { rs = s2; }
+    if (s2 < s1)  { rs = s1; }
+
+    if (e1 == e2) { re = e1; }
+    if (e1 < e2)  { re = e1; }
+    if (e2 < e1)  { re = e2; }
 
 /* rb and re represent resulting address range boundaries */
 
-    libfwbuilder::_convert_range_to_networks(rs,re,res);
+    libfwbuilder::_convert_range_to_networks(rs, re, res);
 
     return res;
 }
 
 vector<InetAddrMask> libfwbuilder::substract(const InetAddrMask &n1,
-                                          const InetAddrMask &n2)
+                                             const InetAddrMask &n2)
 {
     const InetAddr n1s = *(n1.getAddressPtr());
     const InetAddr n2s = *(n2.getAddressPtr());
