@@ -4,9 +4,9 @@
 
                  Copyright (C) 2000 NetCitadel, LLC
 
-  Author:  Vadim Kurland     vadim@vk.crocodile.org
+  Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ObjectGroup.h 177 2008-08-23 23:48:45Z vadim $
+  $Id$
 
 
   This program is free software which we release under the GNU General Public
@@ -38,11 +38,13 @@ class ObjectGroup : public Group
     public:
 
     ObjectGroup();
-    ObjectGroup(const FWObject *root,bool prepopulate);
+    ObjectGroup(const FWObjectDatabase *root,bool prepopulate);
     virtual ~ObjectGroup();
 
     DECLARE_FWOBJECT_SUBTYPE(ObjectGroup);
 
+    DECLARE_DISPATCH_METHODS(ObjectGroup);
+    
     virtual xmlNodePtr toXML(xmlNodePtr xml_parent_node) throw(FWException);
 
     /*
@@ -50,6 +52,13 @@ class ObjectGroup : public Group
      */
     virtual bool    validateChild(FWObject *o);
 
+    /**
+     * get the list of object type names that can be inserted into
+     * given object group. For example, if the group is ObjectGroup,
+     * then the list will include all objects but not services. This
+     * reflects definition of the group XML element in DTD.
+     */
+    virtual void getAllowedTypesOfChildren(std::list<std::string> &types_list);
 };
 
 }
