@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: ServiceGroup.cpp 488 2009-12-12 04:46:26Z vadim $
+  $Id$
 
 
   This program is free software which we release under the GNU General Public
@@ -45,6 +45,7 @@
 #include <fwbuilder/TagService.h>
 #include <fwbuilder/UserService.h>
 #include <fwbuilder/FWServiceReference.h>
+#include <fwbuilder/RuleSet.h>
 
 #include <iostream>
 
@@ -66,7 +67,8 @@ bool  ServiceGroup::validateChild(FWObject *o)
             Address::cast(o)==NULL &&
             ObjectGroup::cast(o)==NULL &&
             Interval::cast(o)==NULL &&
-            FWObjectReference::cast(o)==NULL);
+            FWObjectReference::cast(o)==NULL &&
+            RuleSet::cast(o)==NULL);
 }
 
 FWReference* ServiceGroup::createRef()
