@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: TCPService.h 236 2008-12-25 20:36:41Z vadim $
+  $Id$
 
 
   This program is free software which we release under the GNU General Public
@@ -57,8 +57,10 @@ public:
 
     DECLARE_FWOBJECT_SUBTYPE(TCPService);
 
+    DECLARE_DISPATCH_METHODS(TCPService);
+    
     virtual std::string getProtocolName();
-    virtual int    getProtocolNumber();
+    virtual int getProtocolNumber() const;
 
 /**
  * this is a shortcut method, it returns true if TCPService object
