@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@fwbuilder.org
 
-  $Id: Compiler.cpp 604 2010-05-12 21:10:31Z vadim $
+  $Id$
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -520,7 +520,7 @@ void Compiler::_expand_addr(Rule *rule, FWObject *s,
  * regular address obejcts. Drop objects that do not match current
  * address family.
  */
-void Compiler::_expandAddressRanges(Rule*, FWObject *re)
+void Compiler::_expandAddressRanges(Rule *rule, FWObject *re)
 {
     list<FWObject*> cl;
     for (FWObject::iterator i1=re->begin(); i1!=re->end(); ++i1) 
@@ -542,6 +542,13 @@ void Compiler::_expandAddressRanges(Rule*, FWObject *re)
                 vector<InetAddrMask> vn = 
                     libfwbuilder::convertAddressRange(a1,a2);
 
+                if (vn.size() == 0)
+                {
+                    abort(rule,
+                        "Address Range object '" + aro->getName() +
+                        "' can not be converted to set of addresses");
+                }
+
                 for (vector<InetAddrMask>::iterator i=vn.begin();
                      i!=vn.end(); i++)
                 {
