@@ -43,6 +43,7 @@
 #include "fwbuilder/Routing.h"
 #include "fwbuilder/Rule.h"
 #include "fwbuilder/Firewall.h"
+#include "fwbuilder/Cluster.h"
 #include "fwbuilder/RuleSet.h"
 #include "fwbuilder/InetAddr.h"
 #include "fwbuilder/IPRoute.h"
@@ -68,7 +69,7 @@ int RoutingCompiler::prolog()
 {
     Compiler::prolog();
     
-    FWObject  *routing = fw->getFirstByType(Routing::TYPENAME);
+    Routing *routing = Routing::cast(fw->getFirstByType(Routing::TYPENAME));
     assert(routing);
 
     combined_ruleset = new Routing();   // combined ruleset
@@ -80,11 +81,25 @@ int RoutingCompiler::prolog()
     combined_ruleset->setName(routing->getName());
     temp_ruleset->setName(routing->getName());
 
+    routing->renumberRules();
+
     list<FWObject*> l = routing->getByType(RoutingRule::TYPENAME);
     for (list<FWObject*>::iterator j=l.begin(); j!=l.end(); ++j) 
     {
 	Rule *r= Rule::cast(*j);
-	if (r->isDisabled()) continue;
+        if (r == NULL) continue; // skip RuleSetOptions object
+
+        /*
+         * do not remove disabled rules just yet because some
+         * compilers might use RuleSet::insertRuleAtTop() and other
+         * similar methods from prolog() or
+         * addPredefinedPolicyRules()() and these methods renumber
+         * rules (labels stop matching rule positions when this is
+         * done because labels are configured in prolog() method of
+         * the base class. See fwbuilder ticket 1173)
+         */
+	//if (r->isDisabled()) continue;
+
 	r->setInterfaceId(-1);
 	r->setLabel( createRuleLabel("", "main", r->getPosition()) );
 	combined_ruleset->add( r );
@@ -171,8 +186,8 @@ bool RoutingCompiler::ExpandMultipleAddresses::processNext()
     RuleElementRDst *dst = rule->getRDst();    assert(dst);
     RuleElementRGtw *gtw = rule->getRGtw();    assert(gtw);
 
-    compiler->_expandAddr(rule, dst);
-    compiler->_expandAddr(rule, gtw);
+    compiler->_expand_addr(rule, dst, true);
+    compiler->_expand_addr(rule, gtw, false);
     return true;
 }
 
@@ -230,9 +245,8 @@ bool RoutingCompiler::emptyRDstAndRItf::processNext()
          (FWReference::cast(gtwrel->front())->getPointer())->getName()=="Any")
     {
         string msg;
-        msg = "Gateway and interface are both empty in the rule " +
-            rule->getLabel();
-        compiler->abort( msg.c_str() );
+        msg = "Gateway and interface are both empty in the rule";
+        compiler->abort(rule, msg.c_str());
     }
 
     return true;
@@ -251,10 +265,10 @@ bool RoutingCompiler::singleAdressInRGtw::processNext()
     if( gtwrel->checkSingleIPAdress(o) == false)
     {       
         string msg;
-        msg = "The object \"" + o->getName() + "\" used as gateway in the routing rule "\
-            + rule->getLabel() + " has more than one interface resp. IP adress!";
-        compiler->abort( msg.c_str() );
-        
+        msg = "Object \"" + o->getName() +
+            "\" used as a gateway in the routing rule " +
+            rule->getLabel() + " has multiple ip adresses";
+        compiler->abort(rule, msg.c_str());
     }
     return true;
 }
@@ -307,10 +321,10 @@ bool RoutingCompiler::validateNetwork::processNext()
     if( checkValidNetwork(o) == false) {
     
         string msg;
-        msg = "The object \"" + o->getName() + "\" used as destination in the routing rule "\
-            + rule->getLabel() + " has an invalid netmask!";
-        compiler->abort( msg.c_str() );
-        
+        msg = "Object \"" + o->getName() +
+            "\" used as destination in the routing rule " +
+            rule->getLabel() + " has invalid netmask";
+        compiler->abort(rule, msg.c_str());
     }
     return true;
     
@@ -320,15 +334,16 @@ bool RoutingCompiler::validateNetwork::processNext()
 bool RoutingCompiler::reachableAddressInRGtw::checkReachableIPAddress(FWObject *o)
 {
     // let's walk over all interfaces of this firewall
-    FWObjectTypedChildIterator j = compiler->fw->findByType(Interface::TYPENAME);
+    list<FWObject*> interfaces = compiler->fw->getByTypeDeep(Interface::TYPENAME);
+    list<FWObject*>::iterator intf;
 
     if( Host::cast(o) != NULL)
     {
         Host *host=Host::cast(o);
         const InetAddr *ip_host = host->getAddressPtr();
-        for ( ; j!=j.end(); ++j )
+        for (intf = interfaces.begin(); intf!=interfaces.end(); ++intf)
         {
-            Interface *i_firewall=Interface::cast(*j);
+            Interface *i_firewall = Interface::cast(*intf);
             for(FWObjectTypedChildIterator fw_ips = 
                     i_firewall->findByType(IPv4::TYPENAME);
                 fw_ips!=fw_ips.end(); ++fw_ips)
@@ -353,9 +368,9 @@ bool RoutingCompiler::reachableAddressInRGtw::checkReachableIPAddress(FWObject *
         const InetAddr *ip_gateway = gw_interface->getAddressPtr();
 
         // walk over all interfaces of this firewall
-        for ( ; j!=j.end(); ++j )
+        for (intf = interfaces.begin(); intf!=interfaces.end(); ++intf)
         {
-            Interface *if_firewall=Interface::cast(*j);
+            Interface *if_firewall=Interface::cast(*intf);
             FWObjectTypedChildIterator addresses =
                 if_firewall->findByType(IPv4::TYPENAME);
 
@@ -380,9 +395,9 @@ bool RoutingCompiler::reachableAddressInRGtw::checkReachableIPAddress(FWObject *
         IPv4 *ipv4=IPv4::cast(o);
         const InetAddr *ip_ipv4 = ipv4->getAddressPtr();
 
-        for ( ; j!=j.end(); ++j )
+        for (intf = interfaces.begin(); intf!=interfaces.end(); ++intf)
         {
-            Interface *if_firewall=Interface::cast(*j);
+            Interface *if_firewall=Interface::cast(*intf);
             FWObjectTypedChildIterator addresses =
                 if_firewall->findByType(IPv4::TYPENAME);
 
@@ -418,11 +433,11 @@ bool RoutingCompiler::reachableAddressInRGtw::processNext()
     if( checkReachableIPAddress(o) == false)
     {
         string msg;
-        msg = "The object \"" + o->getName() +
-            "\" used as gateway in the routing rule "\
-            + rule->getLabel() +
-            " is not reachable, since not in any local network of the firewall";
-        compiler->abort( msg.c_str() );
+        msg = "Object \"" + o->getName() +
+            "\" used as gateway in the routing rule " +
+            rule->getLabel() +
+            " is not reachable because it is not in any local network of the firewall";
+        compiler->abort(rule, msg.c_str());
     }
     return true;
 }
@@ -477,10 +492,12 @@ bool RoutingCompiler::contradictionRGtwAndRItf::processNext()
         }
 
         string msg;
-        msg = "The object \"" + oRGtw->getName() + "\" used as gateway in the routing rule "\
-            + rule->getLabel() + " is not in the same local network as the interface " + oRItf->getName() + "!";
-        compiler->abort( msg.c_str() );
-
+        msg = "Object \"" + oRGtw->getName() +
+            "\" used as gateway in the routing rule " +
+            rule->getLabel() +
+            " is not in the same local network as interface " +
+            oRItf->getName();
+        compiler->abort(rule, msg.c_str());
     }
 
     return true;
@@ -493,17 +510,35 @@ bool RoutingCompiler::rItfChildOfFw::processNext()
     tmp_queue.push_back(rule);
     
     RuleElementRItf *itfrel = rule->getRItf();
-    FWObject *o = FWReference::cast(itfrel->front())->getPointer();
 
-    if (!o->isChildOf(compiler->fw))
+    if (itfrel->isAny()) return true;
+
+    FWObject *o = FWReference::cast(itfrel->front())->getPointer();
+    if (o->isChildOf(compiler->fw)) return true;
+
+    // the interface is not a child of the firewall. Could be
+    // cluster interface though. In that case make sure the
+    // firewall is a member of that cluster.
+    Interface *iface = Interface::cast(o);
+    if (iface)
     {
-        string msg;
-        msg = "The object \"" + o->getName() + 
-            "\" used as interface in the routing rule " +
-            rule->getLabel() +
-            " is not a child of the firewall the rule belongs to";
-        compiler->abort( msg.c_str() );
+        Cluster *cluster = Cluster::cast(iface->getParentHost());
+        if (cluster)
+        {            
+            list<Firewall*> members;
+            cluster->getMembersList(members);
+            if (std::find(members.begin(), members.end(), compiler->fw) != members.end())
+                return true;
+        }
     }
+    string msg;
+    msg = "Object \"" + o->getName() + 
+        "\" used as interface in the routing rule " +
+        rule->getLabel() +
+        " is not a child of the firewall the rule belongs to";
+    compiler->abort(rule, msg.c_str());
+    // even though we call abort() here, it does not actually stop the
+    // program if it runs in the test mode.
     return true;
 }
 
@@ -525,7 +560,11 @@ bool RoutingCompiler::competingRules::processNext()
     ostr << gtw->getId() << "_" << itf->getId();
     string combiId = ostr.str();
     
-    if( label == "") compiler->abort("Place 'createSortedDstIdsLabel()' before 'competingRules()' in the rule processor chain!");
+    if( label == "") compiler->abort(
+        
+            rule,         
+            "Place 'createSortedDstIdsLabel()' before 'competingRules()' "
+            "in the rule processor chain");
     
     dest_it = rules_seen_so_far.find(label);
     if( dest_it != rules_seen_so_far.end()) {
@@ -534,9 +573,10 @@ bool RoutingCompiler::competingRules::processNext()
         ///std::cout << "NO NEW DEST" << std::endl;
         
         gtwitf_it = dest_it->second.find(combiId);
-        if( gtwitf_it != dest_it->second.end() ) {
-            
-            // ... this gateway and interface combination were already seen for this destination
+        if( gtwitf_it != dest_it->second.end() )
+        {
+            // ... this gateway and interface combination were already
+            // seen for this destination
             ///std::cout << "NO NEW GTWITF" << std::endl;
             
             if( gtwitf_it->second.first == metric) {
@@ -545,40 +585,44 @@ bool RoutingCompiler::competingRules::processNext()
                 ///std::cout << "SAME METRIC" << std::endl;
                 
                 string msg;
-                msg = "The routing rules " + gtwitf_it->second.second +
+                msg = "Routing rules " + gtwitf_it->second.second +
                     " and " + rule->getLabel() +
-                    " are identical, skipping the second. " +
-                    "Please delete one of them to avoid this warning!";
-                compiler->warning( msg.c_str() );
-            
+                    " are identical, skipping the second one. " +
+                    "Delete one of them to avoid this warning";
+                compiler->warning(rule,  msg.c_str());
             } else {
                 
                 // ... but different metric => what metric should I use? => abort
                 ///std::cout << "DIFFERENT METRIC" << std::endl;
                 
                 string msg;
-                msg = "The routing rules " + gtwitf_it->second.second +
+                msg = "Routing rules " + gtwitf_it->second.second +
                     " and " + rule->getLabel() +
                     " are identical except for the metric, " +
-                    "please delete one of them!";
-                compiler->abort( msg.c_str() );
+                    "please delete one of them";
+                compiler->abort(rule,  msg.c_str());
             }
         
-        } else {
-
-            // ... this gateway and interface combination is new for this destination
+        } else
+        {
+            // ... this gateway and interface combination is new for
+            // this destination
             ///std::cout << "NEW GTWITF" << std::endl;///
             
-            if(false) {
+            if(false)
+            {
                 // TODO_lowPrio: if ( !compiler->fw->getOptionsObject()->getBool ("equal_cost_multi_path") ) ...If multipath is turned off, perform this check.
                 //               iterate all gtwitf combis in the map dest_it->second and search for the current metric
                 
                 // ... but has the same metric => what route should I use for this destination? => abort
                     
                 string msg;
-                msg = "The routing rules " + gtwitf_it->second.second + " and " + rule->getLabel() + " have the same destination,\
-                    \na different gateway and interface combination, but the same metric.\
-                    \nPlease set the metrics to different values or enable ECMP (Equal Cost MultiPath) routing!";
+                msg = "Routing rules " + gtwitf_it->second.second + " and " +
+                    rule->getLabel() +
+                    " have the same destination and same metric,"
+                    "but different gateway and interface combination. "
+                    "Set the metrics to different values or "
+                    "enable ECMP (Equal Cost MultiPath) routing";
                 compiler->abort( msg.c_str() );
             
             } else {
@@ -624,8 +668,8 @@ bool RoutingCompiler::classifyRoutingRules::processNext()
     if (tmp_queue.size()==0) return false;
     
     
-    for (std::deque<libfwbuilder::Rule *>::iterator tmp_queue_it=tmp_queue.begin(); tmp_queue_it!=tmp_queue.end(); ++tmp_queue_it) {
-        
+    for (std::deque<libfwbuilder::Rule *>::iterator tmp_queue_it=tmp_queue.begin(); tmp_queue_it!=tmp_queue.end(); ++tmp_queue_it)
+    {
         RoutingRule *rule = RoutingRule::cast( *tmp_queue_it);
         rule->setRuleType( RoutingRule::SinglePath);
         
@@ -641,7 +685,12 @@ bool RoutingCompiler::classifyRoutingRules::processNext()
         ostr << gtw->getId() << "_" << itf->getId();
         string combiId = ostr.str();
         
-        if( label == "") compiler->abort("Place 'createSortedDstIdsLabel()' right before 'classifyRoutingRules()' in the rule processor chain!");
+        if( label == "")
+            compiler->abort(
+                
+                    rule,
+                    "Place 'createSortedDstIdsLabel()' right before "
+                    "'classifyRoutingRules()' in the rule processor chain");
         
         dest_it = rules_seen_so_far.find(label);
         if( dest_it != rules_seen_so_far.end()) {
